local addonName, addon = ...

local defaults = {
	class = true
}

--[[-----------------------------------------------------------------------------
RegisterDefaultSetting - Allow modules to set their defaults
-------------------------------------------------------------------------------]]
function addon:RegisterDefaultSetting(key, value)	-- Use caution if using this after ADDON_LOADED
	defaults[key] = value
end

--[[-----------------------------------------------------------------------------
Load settings, apply version conversions as needed, and link to defaults
-------------------------------------------------------------------------------]]
addon.RegisterEvent("Settings-Load", 'ADDON_LOADED', function(self, event, name)
	if name ~= addonName then return end
	addon.UnregisterEvent(self, event)
	
	local name, class = UnitClass('player')
	name = UnitName('player')
	local realm = GetRealmName()

	local settings = _G[addonName .. "Settings"]
	if type(settings) ~= 'table' then
		settings = { }
		_G[addonName .. "Settings"] = settings
	end
	if type(settings[realm]) ~= 'table' then
		settings[realm] = { }
	end
	if type(settings[realm][name]) ~= 'table' then
		settings[realm][name] = { class = class }
	end

	--settings.version = nil		-- Reset value on logout (so can iterate over data without issue)
	addon.settings = setmetatable(settings[realm][name], { __index = defaults })
	return settings
end)

--[[-----------------------------------------------------------------------------
Character Specific WTF Reset.
-------------------------------------------------------------------------------]]
function addon:ResetWTF()
	local name, class = UnitClass('player')
	name = UnitName('player')
	local realm = GetRealmName()
	local settings = _G[addonName .. "Settings"]
	
	if type(settings[realm][name]) == 'table' then
		settings[realm][name] = { class = class }
	end
	settings.version = nil		-- Reset value on logout 
	return settings 
end

--[[-----------------------------------------------------------------------------
Remove unused settings or settings that are the same as defaults
-------------------------------------------------------------------------------]]
addon.RegisterEvent("Settings-Unload", 'PLAYER_LOGOUT', function()
	local settings = addon.settings
	for key, value in pairs(settings) do
		if value == defaults[key] or defaults[key] == nil then
			settings[key] = nil
		end
	end
	_G[addonName .. "Settings"].version = GetAddOnMetadata(addonName, 'Version')
end)

--[[-----------------------------------------------------------------------------
LOD Configuration
-------------------------------------------------------------------------------]]
local LOA = LibStub and LibStub('LibOptionsAssist-1.0', true)
if not (LOA and select(2, GetAddOnInfo(addonName .. "_Config"))) then return end	-- Make sure config support exists

addon.configPanel = LOA:AddEntry(addonName, nil, addonName .. "_Config")

_G['SLASH_' .. addonName .. 1] = '/' .. addonName
_G['SLASH_' .. addonName .. 2] = '/' .. addonName:lower()
_G['SLASH_' .. addonName .. 3] = '/' .. addonName:upper()
SlashCmdList[addonName] = addon.configPanel

InterfaceOptionsFrame:SetFrameStrata('HIGH')
InterfaceOptionsFrame.SetFrameStrata = addon.DoNothing

--[[-----------------------------------------------------------------------------
GrimUI Options Frame Logo
-------------------------------------------------------------------------------]]
local logo = CreateFrame('Frame', nil, InterfaceOptionsFrame)
logo:SetHeight(256)
logo:SetWidth(256)
logo:SetPoint('BOTTOM', InterfaceOptionsFrame, 'TOP', 0, -75)
logo:SetBackdrop({
	bgFile = [[Interface\AddOns\GrimUI\Media\GrimUILogo]],
	edgeFile = nil,
	edgeSize = 0,
	tile = false,
	insets = { left = 0, right = 0, top = 0, bottom = 0 }
})
logo:Hide()

--[[-----------------------------------------------------------------------------
In Game Help System Setup
-------------------------------------------------------------------------------]]
function addon:LoadGUIHelp()
	addon.configPanel()
	GUIHelpInitiate()
end

local helpButton = CreateFrame('button', nil, InterfaceOptionsFrame)
helpButton:SetHeight(40)
helpButton:SetWidth(25)
helpButton:SetPoint('TOP', InterfaceOptionsFrame, 'TOP', 190, -37)
helpButton:RegisterForClicks("AnyUp")
helpButton:SetFrameStrata("DIALOG")
helpButton:SetNormalTexture([[Interface\BUTTONS\UI-MicroButton-Help-Up]])
helpButton:SetHighlightTexture([[Interface\BUTTONS\UI-MicroButton-Hilight]])
helpButton:SetPushedTexture([[Interface\BUTTONS\UI-MicroButton-Help-Down]])
local HelpText = helpButton:CreateFontString(nil, 'DIALOG')
HelpText:SetFont([[Fonts\FRIZQT__.TTF]], 14)
HelpText:SetText("Help")
HelpText:SetPoint("RIGHT", helpButton, "LEFT", 0, -5)
HelpText:SetTextColor(1, .10, .10, 1)

helpButton:SetScript('OnClick', function(self)
	
	if button == 'LeftButton' then
		GUIHelpInitiate()
	else
		GUIHelpInitiate()
	end
	
end)
helpButton:Hide()

--[[-------------------------------------------------------------------------------
Config Panel Show/Hide Scripts - Put all config panel adjustment functions here!
---------------------------------------------------------------------------------]]
addon.configPanel:SetScript('OnHide', function()
	InterfaceOptionsFrame:SetWidth(648)
	logo:Hide()
	helpButton:Hide()
end)

addon.configPanel:SetScript('OnShow', function()
	InterfaceOptionsFrame:SetWidth(800)
	logo:Show()
	helpButton:Show()
end)

--[[-------------------------------------------------------------------------------
Default WTF Settings - Try them here to stop nextto error and other errors.
---------------------------------------------------------------------------------]]
--TargetFrame.lua settings
addon:RegisterDefaultSetting("hideBlizTargetFrame", true)

--Skins.lua settings
addon:RegisterDefaultSetting("useViewPort", false)
addon:RegisterDefaultSetting("artSetPoint", 0)
addon:RegisterDefaultSetting("artSetAlpha", 1)

--ChatFrames.lua settings
addon:RegisterDefaultSetting("chatInitialized", false)
addon:RegisterDefaultSetting("showCombatLog", true)
addon:RegisterDefaultSetting("showInfoLog", true)
addon:RegisterDefaultSetting("showChatTabs", false)
addon:RegisterDefaultSetting("resolution", false)

--DashBoard.lua settings
addon:RegisterDefaultSetting("autoRepair", true)
addon:RegisterDefaultSetting("mailShow", true)
addon:RegisterDefaultSetting("durabilityShow", true)
addon:RegisterDefaultSetting("backpackShow", true)
addon:RegisterDefaultSetting("moneyShow", true)
addon:RegisterDefaultSetting("clockShow", true)
addon:RegisterDefaultSetting("latencyShow", true)
addon:RegisterDefaultSetting("fpsShow", true)
addon:RegisterDefaultSetting("socialShow", true)
addon:RegisterDefaultSetting("guildShow", true)
addon:RegisterDefaultSetting("speedShow", true)
addon:RegisterDefaultSetting("clock24hr", false)
addon:RegisterDefaultSetting("GuildRepair", true)
addon:RegisterDefaultSetting("dashboardScale", 1)
addon:RegisterDefaultSetting("CurrentMail", 0)

--MiniMap.lua settings
addon:RegisterDefaultSetting("NotSoMinimap", false)
addon:RegisterDefaultSetting("ShowMiniMap", true)
addon:RegisterDefaultSetting("MapArt", true)
addon:RegisterDefaultSetting("MapButStyle", true)
addon:RegisterDefaultSetting("GmapMask", "Interface\\BUTTONS\\WHITE8X8")
addon:RegisterDefaultSetting("NSmapMask", "Interface\\BUTTONS\\WHITE8X8")
addon:RegisterDefaultSetting("GmapAlpha", 1)
addon:RegisterDefaultSetting("NSmapAlpha", 1)
addon:RegisterDefaultSetting("NSmapScale", 1.5)

--ActionBars.lua settings

--PlayerFrame.lua settings
addon:RegisterDefaultSetting("hideBlizPlayerFrame", true)
addon:RegisterDefaultSetting("showPlayerFrame", true)
addon:RegisterDefaultSetting("playerFrameScale", 1)
addon:RegisterDefaultSetting("playerHealthColorR", 0.231)
addon:RegisterDefaultSetting("playerHealthColorG", 0.682)
addon:RegisterDefaultSetting("playerHealthColorB", 0.419)
addon:RegisterDefaultSetting("playerHealthColorAlpha", 1)
addon:RegisterDefaultSetting("playerHealthTxtColorR", 1)
addon:RegisterDefaultSetting("playerHealthTxtColorG", 1)
addon:RegisterDefaultSetting("playerHealthTxtColorB", 1)
addon:RegisterDefaultSetting("playerHealthTxtColorAlpha", 1)

--ExpRepBar.lua settings
addon:RegisterDefaultSetting("showExpBar", true)
addon:RegisterDefaultSetting("showRepBar", true)
addon:RegisterDefaultSetting("xpBarvar", "xpTot")
addon:RegisterDefaultSetting("repBarvar", "repTot")
addon:RegisterDefaultSetting("showGuild", false)

--ContextMenu.lua settings
addon:RegisterDefaultSetting("showContextMenu", false)

--PartyFrames.lua settings
addon:RegisterDefaultSetting("hideBlizPartyFrames", true)
addon:RegisterDefaultSetting("showPartyFrames", true)
addon:RegisterDefaultSetting("hideGrimPartyInRaid", true)

--StylePartyFrames.lua settings
addon:RegisterDefaultSetting("stylePartyFrames", 1)

--HideFrames.lua

--MoveFrames.lua
addon:RegisterDefaultSetting("lockFrames", false)

--TooltipNotes.lua settings

--Backpack.lua settings

--Tooltips.lua settings
addon:RegisterDefaultSetting("tooltipIconSize", 20)
addon:RegisterDefaultSetting("tooltipAnchor", false)
addon:RegisterDefaultSetting("showToT", true)
addon:RegisterDefaultSetting("showTitle", true)
addon:RegisterDefaultSetting("showRaidIcon", true)
addon:RegisterDefaultSetting("showHealthBar", true)
addon:RegisterDefaultSetting("growDirection", 1)
addon:RegisterDefaultSetting("showSpellIcon", true)
addon:RegisterDefaultSetting("showItemIcon", true)

--DevTools.lua settings
addon:RegisterDefaultSetting("toggleTaintLog", false)
addon:RegisterDefaultSetting("showDevBar", false)

--Hud.lua settings
addon:RegisterDefaultSetting("hudShow", true)
addon:RegisterDefaultSetting("hudWidth", 100)
addon:RegisterDefaultSetting("hudHeight", 100)
addon:RegisterDefaultSetting("hudScale", 1)

--CastBar.lua settings

--Features_A.lua settings
addon:RegisterDefaultSetting("showBagBar", true)
addon:RegisterDefaultSetting("showMicroBar", true)
addon:RegisterDefaultSetting("showWatchFrame", true)
addon:RegisterDefaultSetting("HelpViewed", false)

--Features_B.lua settings